@@ -4,11 +4,11 @@ namespace PhpParser\Node\Stmt;
 
 use PhpParser\Node\Stmt;
 
-/**
- * @property string $remaining Remaining text after halt compiler statement.
- */
 class HaltCompiler extends Stmt
 {
+    /** @var string Remaining text after halt compiler statement. */
+    public $remaining;
+
     /**
      * Constructs a __halt_compiler node.
      *
@@ -16,11 +16,11 @@ class HaltCompiler extends Stmt
      * @param array  $attributes Additional attributes
      */
     public function __construct($remaining, array $attributes = array()) {
-        parent::__construct(
-            array(
-                'remaining' => $remaining,
-            ),
-            $attributes
-        );
+        parent::__construct($attributes);
+        $this->remaining = $remaining;
     }
-}
\ No newline at end of file
+
+    public function getSubNodeNames() {
+        return array('remaining');
+    }
+}
