@@ -16,7 +16,7 @@ class Unset_ extends Node\Stmt
      * @param array       $attributes Additional attributes
      */
     public function __construct(array $vars, array $attributes = array()) {
-        parent::__construct(null, $attributes);
+        parent::__construct($attributes);
         $this->vars = $vars;
     }
 
