@@ -3,7 +3,7 @@
 namespace PhpParser\Lexer;
 
 use PhpParser\LexerTest;
-use PhpParser\Parser;
+use PhpParser\Parser\Tokens;
 
 require_once __DIR__ . '/../LexerTest.php';
 
@@ -31,28 +31,28 @@ class EmulativeTest extends LexerTest
         $lexer = $this->getLexer();
         $lexer->startLexing('<?php ->' . $keyword);
 
-        $this->assertSame(Parser::T_OBJECT_OPERATOR, $lexer->getNextToken());
-        $this->assertSame(Parser::T_STRING, $lexer->getNextToken());
+        $this->assertSame(Tokens::T_OBJECT_OPERATOR, $lexer->getNextToken());
+        $this->assertSame(Tokens::T_STRING, $lexer->getNextToken());
         $this->assertSame(0, $lexer->getNextToken());
     }
 
     public function provideTestReplaceKeywords() {
         return array(
             // PHP 5.5
-            array('finally',       Parser::T_FINALLY),
-            array('yield',         Parser::T_YIELD),
+            array('finally',       Tokens::T_FINALLY),
+            array('yield',         Tokens::T_YIELD),
 
             // PHP 5.4
-            array('callable',      Parser::T_CALLABLE),
-            array('insteadof',     Parser::T_INSTEADOF),
-            array('trait',         Parser::T_TRAIT),
-            array('__TRAIT__',     Parser::T_TRAIT_C),
+            array('callable',      Tokens::T_CALLABLE),
+            array('insteadof',     Tokens::T_INSTEADOF),
+            array('trait',         Tokens::T_TRAIT),
+            array('__TRAIT__',     Tokens::T_TRAIT_C),
 
             // PHP 5.3
-            array('__DIR__',       Parser::T_DIR),
-            array('goto',          Parser::T_GOTO),
-            array('namespace',     Parser::T_NAMESPACE),
-            array('__NAMESPACE__', Parser::T_NS_C),
+            array('__DIR__',       Tokens::T_DIR),
+            array('goto',          Tokens::T_GOTO),
+            array('namespace',     Tokens::T_NAMESPACE),
+            array('__NAMESPACE__', Tokens::T_NS_C),
         );
     }
 
@@ -80,7 +80,7 @@ class EmulativeTest extends LexerTest
         $lexer = $this->getLexer();
         $lexer->startLexing('<?php ' . $stringifiedToken);
 
-        $this->assertSame(Parser::T_CONSTANT_ENCAPSED_STRING, $lexer->getNextToken($text));
+        $this->assertSame(Tokens::T_CONSTANT_ENCAPSED_STRING, $lexer->getNextToken($text));
         $this->assertSame($stringifiedToken, $text);
         $this->assertSame(0, $lexer->getNextToken());
     }
@@ -88,44 +88,44 @@ class EmulativeTest extends LexerTest
     public function provideTestLexNewFeatures() {
         return array(
             array('yield from', array(
-                array(Parser::T_YIELD_FROM, 'yield from'),
+                array(Tokens::T_YIELD_FROM, 'yield from'),
             )),
             array("yield\r\nfrom", array(
-                array(Parser::T_YIELD_FROM, "yield\r\nfrom"),
+                array(Tokens::T_YIELD_FROM, "yield\r\nfrom"),
             )),
             array('...', array(
-                array(Parser::T_ELLIPSIS, '...'),
+                array(Tokens::T_ELLIPSIS, '...'),
             )),
             array('**', array(
-                array(Parser::T_POW, '**'),
+                array(Tokens::T_POW, '**'),
             )),
             array('**=', array(
-                array(Parser::T_POW_EQUAL, '**='),
+                array(Tokens::T_POW_EQUAL, '**='),
             )),
             array('??', array(
-                array(Parser::T_COALESCE, '??'),
+                array(Tokens::T_COALESCE, '??'),
             )),
             array('<=>', array(
-                array(Parser::T_SPACESHIP, '<=>'),
+                array(Tokens::T_SPACESHIP, '<=>'),
             )),
             array('0b1010110', array(
-                array(Parser::T_LNUMBER, '0b1010110'),
+                array(Tokens::T_LNUMBER, '0b1010110'),
             )),
             array('0b1011010101001010110101010010101011010101010101101011001110111100', array(
-                array(Parser::T_DNUMBER, '0b1011010101001010110101010010101011010101010101101011001110111100'),
+                array(Tokens::T_DNUMBER, '0b1011010101001010110101010010101011010101010101101011001110111100'),
             )),
             array('\\', array(
-                array(Parser::T_NS_SEPARATOR, '\\'),
+                array(Tokens::T_NS_SEPARATOR, '\\'),
             )),
             array("<<<'NOWDOC'\nNOWDOC;\n", array(
-                array(Parser::T_START_HEREDOC, "<<<'NOWDOC'\n"),
-                array(Parser::T_END_HEREDOC, 'NOWDOC'),
+                array(Tokens::T_START_HEREDOC, "<<<'NOWDOC'\n"),
+                array(Tokens::T_END_HEREDOC, 'NOWDOC'),
                 array(ord(';'), ';'),
             )),
             array("<<<'NOWDOC'\nFoobar\nNOWDOC;\n", array(
-                array(Parser::T_START_HEREDOC, "<<<'NOWDOC'\n"),
-                array(Parser::T_ENCAPSED_AND_WHITESPACE, "Foobar\n"),
-                array(Parser::T_END_HEREDOC, 'NOWDOC'),
+                array(Tokens::T_START_HEREDOC, "<<<'NOWDOC'\n"),
+                array(Tokens::T_ENCAPSED_AND_WHITESPACE, "Foobar\n"),
+                array(Tokens::T_END_HEREDOC, 'NOWDOC'),
                 array(ord(';'), ';'),
             )),
         );
