@@ -2,6 +2,8 @@
 
 namespace PhpParser;
 
+use PhpParser\Parser\Tokens;
+
 class LexerTest extends \PHPUnit_Framework_TestCase
 {
     /* To allow overwriting in parent class */
@@ -12,28 +14,36 @@ class LexerTest extends \PHPUnit_Framework_TestCase
     /**
      * @dataProvider provideTestError
      */
-    public function testError($code, $message) {
+    public function testError($code, $messages) {
         if (defined('HHVM_VERSION')) {
             $this->markTestSkipped('HHVM does not throw warnings from token_get_all()');
         }
 
-        $lexer = $this->getLexer();
-        try {
-            $lexer->startLexing($code);
-        } catch (Error $e) {
-            $this->assertSame($message, $e->getMessage());
+        $errorHandler = new ErrorHandler\Collecting();
+        $lexer = $this->getLexer(['usedAttributes' => [
+            'comments', 'startLine', 'endLine', 'startFilePos', 'endFilePos'
+        ]]);
+        $lexer->startLexing($code, $errorHandler);
+        $errors = $errorHandler->getErrors();
 
-            return;
+        $this->assertSame(count($messages), count($errors));
+        for ($i = 0; $i < count($messages); $i++) {
+            $this->assertSame($messages[$i], $errors[$i]->getMessageWithColumnInfo($code));
         }
-
-        $this->fail('Expected PhpParser\Error');
     }
 
     public function provideTestError() {
         return array(
-            array('<?php /*', 'Unterminated comment on line 1'),
-            array('<?php ' . "\1", 'Unexpected character "' . "\1" . '" (ASCII 1) on unknown line'),
-            array('<?php ' . "\0", 'Unexpected null byte on unknown line'),
+            array("<?php /*", array("Unterminated comment from 1:7 to 1:9")),
+            array("<?php \1", array("Unexpected character \"\1\" (ASCII 1) from 1:7 to 1:7")),
+            array("<?php \0", array("Unexpected null byte from 1:7 to 1:7")),
+            // Error with potentially emulated token
+            array("<?php ?? \0", array("Unexpected null byte from 1:10 to 1:10")),
+            array("<?php\n\0\1 foo /* bar", array(
+                "Unexpected null byte from 2:1 to 2:1",
+                "Unexpected character \"\1\" (ASCII 1) from 2:2 to 2:2",
+                "Unterminated comment from 2:8 to 2:14"
+            )),
         );
     }
 
@@ -61,7 +71,7 @@ class LexerTest extends \PHPUnit_Framework_TestCase
                 array(),
                 array(
                     array(
-                        Parser::T_STRING, 'tokens',
+                        Tokens::T_STRING, 'tokens',
                         array('startLine' => 1), array('endLine' => 1)
                     ),
                     array(
@@ -69,8 +79,9 @@ class LexerTest extends \PHPUnit_Framework_TestCase
                         array('startLine' => 1), array('endLine' => 1)
                     ),
                     array(
-                        Parser::T_INLINE_HTML, 'plaintext',
-                        array('startLine' => 1), array('endLine' => 1)
+                        Tokens::T_INLINE_HTML, 'plaintext',
+                        array('startLine' => 1, 'hasLeadingNewline' => false),
+                        array('endLine' => 1)
                     ),
                 )
             ),
@@ -84,14 +95,16 @@ class LexerTest extends \PHPUnit_Framework_TestCase
                         array('startLine' => 2), array('endLine' => 2)
                     ),
                     array(
-                        Parser::T_STRING, 'token',
+                        Tokens::T_STRING, 'token',
                         array('startLine' => 2), array('endLine' => 2)
                     ),
                     array(
                         ord('$'), '$',
                         array(
                             'startLine' => 3,
-                            'comments' => array(new Comment\Doc('/** doc' . "\n" . 'comment */', 2))
+                            'comments' => array(
+                                new Comment\Doc('/** doc' . "\n" . 'comment */', 2, 14),
+                            )
                         ),
                         array('endLine' => 3)
                     ),
@@ -103,14 +116,14 @@ class LexerTest extends \PHPUnit_Framework_TestCase
                 array(),
                 array(
                     array(
-                        Parser::T_STRING, 'token',
+                        Tokens::T_STRING, 'token',
                         array(
                             'startLine' => 2,
                             'comments' => array(
-                                new Comment('/* comment */', 1),
-                                new Comment('// comment' . "\n", 1),
-                                new Comment\Doc('/** docComment 1 */', 2),
-                                new Comment\Doc('/** docComment 2 */', 2),
+                                new Comment('/* comment */', 1, 6),
+                                new Comment('// comment' . "\n", 1, 20),
+                                new Comment\Doc('/** docComment 1 */', 2, 31),
+                                new Comment\Doc('/** docComment 2 */', 2, 50),
                             ),
                         ),
                         array('endLine' => 2)
@@ -123,7 +136,7 @@ class LexerTest extends \PHPUnit_Framework_TestCase
                 array(),
                 array(
                     array(
-                        Parser::T_CONSTANT_ENCAPSED_STRING, '"foo' . "\n" . 'bar"',
+                        Tokens::T_CONSTANT_ENCAPSED_STRING, '"foo' . "\n" . 'bar"',
                         array('startLine' => 1), array('endLine' => 2)
                     ),
                 )
@@ -134,7 +147,7 @@ class LexerTest extends \PHPUnit_Framework_TestCase
                 array('usedAttributes' => array('startFilePos', 'endFilePos')),
                 array(
                     array(
-                        Parser::T_CONSTANT_ENCAPSED_STRING, '"a"',
+                        Tokens::T_CONSTANT_ENCAPSED_STRING, '"a"',
                         array('startFilePos' => 6), array('endFilePos' => 8)
                     ),
                     array(
@@ -142,7 +155,7 @@ class LexerTest extends \PHPUnit_Framework_TestCase
                         array('startFilePos' => 9), array('endFilePos' => 9)
                     ),
                     array(
-                        Parser::T_CONSTANT_ENCAPSED_STRING, '"b"',
+                        Tokens::T_CONSTANT_ENCAPSED_STRING, '"b"',
                         array('startFilePos' => 18), array('endFilePos' => 20)
                     ),
                     array(
@@ -157,7 +170,7 @@ class LexerTest extends \PHPUnit_Framework_TestCase
                 array('usedAttributes' => array('startTokenPos', 'endTokenPos')),
                 array(
                     array(
-                        Parser::T_CONSTANT_ENCAPSED_STRING, '"a"',
+                        Tokens::T_CONSTANT_ENCAPSED_STRING, '"a"',
                         array('startTokenPos' => 1), array('endTokenPos' => 1)
                     ),
                     array(
@@ -165,7 +178,7 @@ class LexerTest extends \PHPUnit_Framework_TestCase
                         array('startTokenPos' => 2), array('endTokenPos' => 2)
                     ),
                     array(
-                        Parser::T_CONSTANT_ENCAPSED_STRING, '"b"',
+                        Tokens::T_CONSTANT_ENCAPSED_STRING, '"b"',
                         array('startTokenPos' => 5), array('endTokenPos' => 5)
                     ),
                     array(
@@ -180,7 +193,7 @@ class LexerTest extends \PHPUnit_Framework_TestCase
                 array('usedAttributes' => array()),
                 array(
                     array(
-                        Parser::T_VARIABLE, '$bar',
+                        Tokens::T_VARIABLE, '$bar',
                         array(), array()
                     ),
                     array(
@@ -188,7 +201,13 @@ class LexerTest extends \PHPUnit_Framework_TestCase
                         array(), array()
                     )
                 )
-            )
+            ),
+            // tests no tokens
+            array(
+                '',
+                array(),
+                array()
+            ),
         );
     }
 
@@ -199,7 +218,7 @@ class LexerTest extends \PHPUnit_Framework_TestCase
         $lexer = $this->getLexer();
         $lexer->startLexing($code);
 
-        while (Parser::T_HALT_COMPILER !== $lexer->getNextToken());
+        while (Tokens::T_HALT_COMPILER !== $lexer->getNextToken());
 
         $this->assertSame($remaining, $lexer->handleHaltCompiler());
         $this->assertSame(0, $lexer->getNextToken());
@@ -215,6 +234,18 @@ class LexerTest extends \PHPUnit_Framework_TestCase
         );
     }
 
+    /**
+     * @expectedException \PhpParser\Error
+     * @expectedExceptionMessage __HALT_COMPILER must be followed by "();"
+     */
+    public function testHandleHaltCompilerError() {
+        $lexer = $this->getLexer();
+        $lexer->startLexing('<?php ... __halt_compiler invalid ();');
+
+        while (Tokens::T_HALT_COMPILER !== $lexer->getNextToken());
+        $lexer->handleHaltCompiler();
+    }
+
     public function testGetTokens() {
         $code = '<?php "a";' . "\n" . '// foo' . "\n" . '"b";';
         $expectedTokens = array(
